/*
 *    This file is part of MixShark.
 *
 *  MixShark is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  MixShark is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MixShark.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include "mainview.h"
#include "maintabbar.h"
#include "tabcontent.h"
#include "webkittc.h"
#include "settingstc.h"
#include "bookmarksviewtc.h"
#include "urlfield.h"
#include "application.h"
#include "menubutton.h"
#include "addtabbutton.h"

#include <QToolButton>
#include <QApplication>
#include <QtDebug>

MainView::MainView(QWidget *parent) :
    QTabWidget(parent)
{
    setTabBar(new MainTabBar);
    m_buttonAddTab = new AddTabButton(tabBar());
    addWebTab();
    setDocumentMode(true);
#ifndef Q_WS_MAC
    menuButton = new MenuButton(this);
    setCornerWidget(menuButton, Qt::TopLeftCorner);
#endif
    updateAddTabButtonPosition();

    connect(this, SIGNAL(currentChanged(int)), SLOT(onCurrentChanged()));
    connect(m_buttonAddTab, SIGNAL(clicked()), this, SLOT(addWebTab()));
    connect(this, SIGNAL(tabCloseRequested(int)), this, SLOT(removeTab(int)));
}

MainTabBar *MainView::tabBar() const { return qobject_cast<MainTabBar *>(QTabWidget::tabBar()); }
TabContent *MainView::currentWidget() const { return qobject_cast<TabContent *>(QTabWidget::currentWidget()); }
TabContent *MainView::widget(int index) const { return qobject_cast<TabContent *>(QTabWidget::widget(index)); }

TabContent *MainView::addWebTab()
{
    return addWebTab(app()->settings().value("HomePage").toUrl());
}

TabContent *MainView::addWebTab(QUrl url)
{
    TabContent *tc = app()->createWebTabContent();
    setCurrentIndex(addTab(tc, ""));
    updateAddTabButtonPosition();
    tc->load(url);

    connect(tc, SIGNAL(titleChanged(QString)), this, SLOT(onTabContentTitleChanged(QString)));
    connect(tc, SIGNAL(loadUrlNewTabRequested(QUrl)), this, SLOT(addWebTab(QUrl)));
    connect(tc, SIGNAL(closeTabRequested()), this, SLOT(onCloseTabRequested()));
    connect(this, SIGNAL(currentTabUrlChanged(QUrl)), tc, SLOT(setUrlFieldValue(QUrl)));
    return tc;
}

TabContent *MainView::addSettingsTab()
{
    TabContent *tc = new SettingsTabContent;
    setCurrentIndex(addTab(tc, ""));
    updateAddTabButtonPosition();

    connect(tc, SIGNAL(loadUrlNewTabRequested(QUrl)), this, SLOT(addWebTab(QUrl)));
    connect(tc, SIGNAL(closeTabRequested()), this, SLOT(onCloseTabRequested()));
    return tc;
}

TabContent *MainView::addBookmarksViewTab()
{
    TabContent *tc = new BookmarksViewTabContent;
    setCurrentIndex(addTab(tc, ""));
    updateAddTabButtonPosition();

    connect(tc, SIGNAL(loadUrlNewTabRequested(QUrl)), this, SLOT(addWebTab(QUrl)));
    connect(tc, SIGNAL(closeTabRequested()), this, SLOT(onCloseTabRequested()));
    return tc;
}

void MainView::removeTab(int index)
{
    if(count() <= 1)
    {
        qApp->quit();
        return;
    }
    QWidget *w = widget(index);
    QTabWidget::removeTab(index);
    delete w;
    updateAddTabButtonPosition();
}

void MainView::onTabContentTitleChanged(QString title)
{
    int senderIndex = indexOf(qobject_cast<QWidget *>(sender()));
    setTabText(senderIndex, title);
    if(senderIndex == currentIndex()) emit currentTabTitleChanged(title);
}

void MainView::onCurrentChanged()
{
    emit currentTabTitleChanged(currentWidget()->title());
}

void MainView::onCloseTabRequested()
{
    removeTab(indexOf(qobject_cast<QWidget *>(sender())));
}

void MainView::updateAddTabButtonPosition()
{
    int posX = 0;
    for(int i = 0; i < count(); i++) posX += tabBar()->tabSizeHint(i).width();

    int tabBarWidth = width()
                      -24 // Add tab button width
#ifndef Q_WS_MAC
                      -64 // Menu button width
#endif
                      ;

    if((width() - 64 - 24) < 192 * count())
        setCornerWidget(m_buttonAddTab);
    else
    {
        setCornerWidget(NULL);
        m_buttonAddTab->setParent(tabBar());
        m_buttonAddTab->move(QPoint(posX, 0));
    }

    m_buttonAddTab->show();
}

void MainView::resizeEvent(QResizeEvent *event)
{
    QTabWidget::resizeEvent(event);
    updateAddTabButtonPosition();
}

void MainView::setMenuButtonMenu(QMenu *newMenu)
{
     menu = newMenu;
     menuButton->setMenu(newMenu);
}
