/*
 *    This file is part of MixShark.
 *
 *  MixShark is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  MixShark is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MixShark.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include "mainwindow.h"
#include "mainview.h"
#include "maintoolbar.h"
#include "urlfield.h"
#include "tabcontent.h"
#include "application.h"
#include "bookmarksmanager.h"
#include "ui_aboutdialog.h"

#include <QLibrary>
#include <QLabel>
#include <QMenu>
#include <QMenuBar>
#include <QShortcut>

#include <QFileDialog>

#ifdef Q_WS_WIN
#include <windows.h>
#include <qt_windows.h>

typedef struct
{
  int cxLeftWidth;
  int cxRightWidth;
  int cyTopHeight;
  int cyBottomHeight;
} MARGINS;

typedef HRESULT (WINAPI *PtrDwmExtendFrameIntoClientArea)(HWND hWnd, const MARGINS *margins);

typedef HRESULT (WINAPI *PtrDwmIsCompositionEnabled)(BOOL *pfEnabled);
#endif

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent), mainView(new MainView)
{
#ifdef Q_WS_MAC
    menus = new QMenuBar;
    setMenuBar(menus);
#else
    menus = new QMenu(this);
    mainView->setMenuButtonMenu(menus);
#endif

    initMenuFile();
    initMenuEdit();
    initMenuBookmarks();
    initMenuHelp();

    setCentralWidget(mainView);
    enableDwmComposition();
    setWindowIcon(QIcon(":/MixSharkIcon.png"));

    connect(mainView, SIGNAL(currentTabTitleChanged(QString)), this, SLOT(setWindowTitle(QString)));
}

MainWindow::~MainWindow()
{
}

void MainWindow::initMenuFile()
{
    menuFile = menus->addMenu(tr("File"));

    actionNewTab = menuFile->addAction(tr("New tab"), mainView, SLOT(addWebTab()));
    actionNewTab->setShortcut(QKeySequence("Ctrl+T"));

    actionCloseTab = menuFile->addAction(tr("Close tab"), mainView, SLOT(removeTab()));
    actionCloseTab->setShortcut(QKeySequence("Ctrl+W"));

    actionOpenFile = menuFile->addAction(tr("Open file"), this, SLOT(openFile()));
    actionOpenFile->setShortcut(QKeySequence::Open);

    actionSaveCurrentPage = menuFile->addAction(tr("Save page"), this, SLOT(saveCurrentPage()));
    actionSaveCurrentPage->setShortcut(QKeySequence::Save);
}

void MainWindow::initMenuEdit()
{
    menuEdit = menus->addMenu(tr("Edit"));

    actionUndo = menuEdit->addAction(tr("Undo"), this, SLOT(undo()));
    actionUndo->setShortcut(QKeySequence::Undo);

    actionRedo = menuEdit->addAction(tr("Redo"), this, SLOT(redo()));
    actionRedo->setShortcut(QKeySequence::Redo);

    menuEdit->addSeparator();

    actionCut = menuEdit->addAction(tr("Cut"), this, SLOT(cut()));
    actionCut->setShortcut(QKeySequence::Cut);

    actionCopy = menuEdit->addAction(tr("Copy"), this, SLOT(copy()));
    actionCopy->setShortcut(QKeySequence::Copy);

    actionPaste = menuEdit->addAction(tr("Paste"), this, SLOT(paste()));
    actionPaste->setShortcut(QKeySequence::Paste);

    actionSelectAll = menuEdit->addAction(tr("Select all"), this, SLOT(selectAll()));
    actionSelectAll->setShortcut(QKeySequence::SelectAll);

    menuEdit->addSeparator();

    actionLoadSettingsDialog = menuEdit->addAction(tr("Settings"), mainView, SLOT(addSettingsTab()));
    actionLoadSettingsDialog->setShortcut(QKeySequence("Ctrl+,"));
}

void MainWindow::initMenuBookmarks()
{
    menuBookmarks = menus->addMenu(tr("Bookmarks"));
    updateMenuBookmarks();
    connect(app()->bookmarksManager(), SIGNAL(bookmarksChanged()), this, SLOT(updateMenuBookmarks()));
}

void MainWindow::initMenuHelp()
{
    menuHelp = menus->addMenu(tr("?"));
    actionAboutMixShark = menuHelp->addAction(tr("About MixShark"), this, SLOT(aboutMixShark()));
}

void MainWindow::updateMenuBookmarks()
{
    menuBookmarks->clear();
    actionAddCurrentPageToBookmarks = menuBookmarks->addAction(tr("Add to bookmarks"), this, SLOT(addCurrentPageToBookmarks()));
    actionAddCurrentPageToBookmarks->setShortcut(QKeySequence("Ctrl+D"));
    actionManageBookmarks = menuBookmarks->addAction(tr("Manage"), mainView, SLOT(addBookmarksViewTab()));
    menuBookmarks->addSeparator();

    QList<Bookmark> bookmarks = app()->bookmarksManager()->getBookmarks();
    int bmCount = bookmarks.count();
    foreach(Bookmark bookmark, bookmarks)
    {
        QAction *a = new QAction(bookmark.second, this);
        a->setProperty("Url", bookmark.first);
        connect(a, SIGNAL(triggered()), this, SLOT(bookmarkTriggered()));
        menuBookmarks->addAction(a);
    }
}

void MainWindow::openFile()
{
    mainView->addWebTab(QFileDialog::getOpenFileName(this, tr("Open file")));
}

void MainWindow::saveCurrentPage()
{
    QFile file(QFileDialog::getSaveFileName(this, tr("Save page")));
    if(file.open(QFile::WriteOnly))
        file.write(mainView->currentWidget()->data().toAscii());
}

void MainWindow::undo()
{
    mainView->currentWidget()->undo();
}

void MainWindow::redo()
{
    mainView->currentWidget()->redo();
}

void MainWindow::cut()
{
    mainView->currentWidget()->cut();
}

void MainWindow::copy()
{
    mainView->currentWidget()->copy();
}

void MainWindow::paste()
{
    mainView->currentWidget()->paste();
}

void MainWindow::selectAll()
{
    mainView->currentWidget()->selectAll();
}

void MainWindow::addCurrentPageToBookmarks()
{
    app()->bookmarksManager()->addBookmark(mainView->currentWidget()->url(), mainView->currentWidget()->title());
}

void MainWindow::bookmarkTriggered()
{
    mainView->addWebTab(sender()->property("Url").toUrl());
}

void MainWindow::aboutMixShark()
{
    QDialog *dialog = new QDialog(this);
    Ui::AboutDialog ui;
    ui.setupUi(dialog);
    ui.title->setText(QString("<h1>MixShark %1</h1>").arg(app()->applicationVersion()));
    dialog->exec();
}

void MainWindow::enableDwmComposition()
{
#ifdef Q_WS_WIN
    QLibrary dwmapi("dwmapi");

    if(QSysInfo::WindowsVersion >= QSysInfo::WV_VISTA)
    {
        BOOL enabled;
        ((PtrDwmIsCompositionEnabled)dwmapi.resolve("DwmIsCompositionEnabled"))(&enabled);

        if(enabled)
        {
            MARGINS margins = {-1};

            RECT rcClient;
            GetWindowRect(winId(), &rcClient);
            SetWindowPos(winId(),
                         NULL,
                         rcClient.left, rcClient.top,
                         rcClient.right-rcClient.left, rcClient.bottom-rcClient.top,
                         SWP_FRAMECHANGED);

            ((PtrDwmExtendFrameIntoClientArea)dwmapi.resolve("DwmExtendFrameIntoClientArea"))(winId(), &margins);
            setAttribute(Qt::WA_TranslucentBackground);
        }
    }

    dwmapi.unload();
#endif
}
