/*
 *    This file is part of MixShark.
 *
 *  MixShark is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  MixShark is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MixShark.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include "tabcontent.h"
#include "maintoolbar.h"
#include "urlfield.h"
#include "progressindicator.h"
#include "htmlhighlighter.h"

#include <QVBoxLayout>
#include <QAction>
#include <QStyle>
#include <QMenu>
#include <QTextEdit>
#include <QDockWidget>

TabContent::TabContent(QWidget *parent) :
    QMainWindow(parent)
{
    sourceCodeEdit = new QTextEdit;
    sourceCodeHighlighter = new HtmlHighlighter(sourceCodeEdit->document());
    dockWidgetSourceCode = new QDockWidget;
    dockWidgetSourceCode->setWidget(sourceCodeEdit);
    addDockWidget(Qt::BottomDockWidgetArea, dockWidgetSourceCode);
    dockWidgetSourceCode->hide();

    actionBack = new QAction(style()->standardPixmap(QStyle::SP_ArrowBack), tr("Back"), this);
    actionBack->setShortcut(QKeySequence::Back);

    actionForward = new QAction(style()->standardPixmap(QStyle::SP_ArrowForward), tr("Forward"), this);
    actionForward->setShortcut(QKeySequence::Forward);

    actionReloadOrStop = new QAction(style()->standardPixmap(QStyle::SP_BrowserReload), tr("Reload"), this);
    actionReloadOrStop->setShortcut(QKeySequence::Refresh);

    actionOpenLink = new QAction(tr("Open link"), this);
    actionOpenLinkNewTab = new QAction(tr("Open link in new tab"), this);

    actionShowImage = new QAction(tr("Show image"), this);

    actionCut = new QAction(tr("Cut"), this);
    actionCut->setShortcut(QKeySequence::Cut);

    actionCopy = new QAction(tr("Copy"), this);
    actionCopy->setShortcut(QKeySequence::Copy);

    actionPaste = new QAction(tr("Paste"), this);
    actionPaste->setShortcut(QKeySequence::Paste);

    actionShowSourceCode = new QAction(tr("Show source code"), this);

    connect(actionBack, SIGNAL(triggered()), this, SLOT(back()));
    connect(actionForward, SIGNAL(triggered()), this, SLOT(forward()));
    connect(actionReloadOrStop, SIGNAL(triggered()), this, SLOT(reload()));

    connect(actionOpenLink, SIGNAL(triggered()), this, SLOT(openActiveLink()));
    connect(actionOpenLinkNewTab, SIGNAL(triggered()), this, SLOT(openActiveLinkNewTab()));

    connect(actionShowImage, SIGNAL(triggered()), this, SLOT(showActiveImage()));

    connect(actionCut, SIGNAL(triggered()), this, SLOT(cut()));
    connect(actionCopy, SIGNAL(triggered()), this, SLOT(copy()));

    connect(actionShowSourceCode, SIGNAL(triggered()), this, SLOT(showSource()));

    connect(this, SIGNAL(urlChanged(QUrl)), this, SLOT(setUrlFieldValue(QUrl)));
    connect(this, SIGNAL(dataChanged()), this, SLOT(updateSourceCodeEdit()));
}

void TabContent::showSource()
{
    dockWidgetSourceCode->show();
}

void TabContent::loadUrlFieldContent()
{
    load(QUrl::fromUserInput(mainToolBar->urlFieldText()));
}

void TabContent::setUrlFieldValue(const QUrl &url)
{
    QString mqldkmlqskdmlq = url.toString();
    mainToolBar->setUrlFieldText(url.toString());
}

void TabContent::requestAddPageToBookmarks()
{
    emit addBookmarkRequested(url());
}

void TabContent::requestSavePageAs()
{
    emit savePageAsRequested(data());
}

void TabContent::requestShowSourceCode()
{
    emit showSourceCodeRequested(data());
}

void TabContent::addNavigationToolBar()
{
    mainToolBar = new MainToolBar(actionBack, actionForward);
    progressIndicator = new ProgressIndicator;
    addToolBar(mainToolBar);
    connect(mainToolBar, SIGNAL(backButtonPressed()), this, SLOT(back()));
    connect(mainToolBar, SIGNAL(forwardButtonPressed()), this, SLOT(forward()));
    connect(mainToolBar, SIGNAL(urlFieldReturnPressed()), this, SLOT(loadUrlFieldContent()));
    mainToolBar->addWidget(progressIndicator);
    connect(this, SIGNAL(loadStarted()), SLOT(onLoadStarted()));
    connect(this, SIGNAL(loadFinished(bool)), SLOT(onLoadFinished()));
}

void TabContent::updateSourceCodeEdit()
{
    sourceCodeEdit->setPlainText(data());
}

void TabContent::onLoadStarted()
{
    progressIndicator->show();
    progressIndicator->startAnimation();
}

void TabContent::onLoadFinished()
{
    bool b = canGoBack(), f = canGoForward();
    if(actionBack) actionBack->setEnabled(b);
    if(actionForward) actionForward->setEnabled(f);

    progressIndicator->hide();
    progressIndicator->stopAnimation();
}
