/*
 *    This file is part of MixShark.
 *
 *  MixShark is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  MixShark is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MixShark.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include "webkittc.h"
#include "webview.h"

#include <QMenu>
#include <QContextMenuEvent>
#include <QWebFrame>
#include <QWebElementCollection>
#include <QAbstractNetworkCache>
#include <QWebHistory>
#include <QWebInspector>
#include <QDockWidget>
#include <QtDebug>

WebKitTabContent::WebKitTabContent(QWidget *parent) :
    TabContent(parent), webView(new WebView(this))
{
    setWidget(webView);

    inspector = new QWebInspector(this);
    inspector->setPage(webView->page());
    dockWidgetInspector = new QDockWidget(tr("Web inspector"));
    dockWidgetInspector->setWidget(inspector);
    addDockWidget(Qt::BottomDockWidgetArea, dockWidgetInspector);
    dockWidgetInspector->hide();

    addNavigationToolBar();

    setAutoFillBackground(true);

    actionInspectElement = new QAction(tr("Inspect element"), this);
    connect(actionInspectElement, SIGNAL(triggered()), this, SLOT(inspectElement()));

    connect(webView, SIGNAL(titleChanged(QString)), this, SIGNAL(titleChanged(QString)));
    connect(webView, SIGNAL(urlChanged(QUrl)), this, SIGNAL(urlChanged(QUrl)));
    connect(webView, SIGNAL(loadStarted()), this, SIGNAL(loadStarted()));
    connect(webView, SIGNAL(loadFinished(bool)), this, SIGNAL(loadFinished(bool)));
    connect(webView->page(), SIGNAL(loadProgress(int)), this, SIGNAL(dataChanged()));
}

void WebKitTabContent::load(const QUrl &url)
{
    webView->load(url);
}

QString WebKitTabContent::title()
{
    return webView->title();
}

QUrl WebKitTabContent::url()
{
    return webView->url();
}

QString WebKitTabContent::data()
{
    return webView->page()->mainFrame()->toHtml();
}

QIcon WebKitTabContent::icon()
{
    QIcon icon;
    QWebElementCollection linkCollection = webView->page()->mainFrame()->findAllElements("link");
    foreach(QWebElement linkElement, linkCollection)
    {
        if(linkElement.attribute("rel").contains("icon"))
        {
            QUrl iconUrl = linkElement.attribute("href");
            if(iconUrl.isRelative())
                iconUrl = url().resolved(iconUrl);

            QAbstractNetworkCache *cache = webView->page()->networkAccessManager()->cache();
            if(cache)
            {
                QScopedPointer<QIODevice> imageFile(cache->data(iconUrl));
                if(imageFile)
                {
                    QPixmap pixmap;
                    if(pixmap.loadFromData(imageFile->readAll()))
                        icon = pixmap;
                }
            }
        }
    }

    return icon;
}

void WebKitTabContent::back()
{
    QUrl backUrl = webView->history()->backItem().url();
    webView->back();
    emit urlChanged(backUrl);
}

void WebKitTabContent::forward()
{
    QUrl forwardUrl = webView->history()->forwardItem().url();
    webView->forward();
    emit urlChanged(forwardUrl);
}

void WebKitTabContent::reload()
{
    webView->reload();
}

void WebKitTabContent::openActiveLink()
{
    webView->load(sender()->property("Url").toUrl());
}

void WebKitTabContent::openActiveLinkNewTab()
{
    emit loadUrlNewTabRequested(sender()->property("Url").toUrl());
}

void WebKitTabContent::showActiveImage()
{
    webView->load(sender()->property("ImageUrl").toUrl());
}

void WebKitTabContent::undo()
{
    webView->page()->triggerAction(QWebPage::Undo);
}

void WebKitTabContent::redo()
{
    webView->page()->triggerAction(QWebPage::Undo);
}

void WebKitTabContent::cut()
{
    webView->page()->triggerAction(QWebPage::Cut);
}

void WebKitTabContent::copy()
{
    webView->page()->triggerAction(QWebPage::Copy);
}

void WebKitTabContent::paste()
{
    webView->page()->triggerAction(QWebPage::Paste);
}

void WebKitTabContent::selectAll()
{
    webView->page()->triggerAction(QWebPage::SelectAll);
}

void WebKitTabContent::inspectElement()
{
    webView->page()->triggerAction(QWebPage::InspectElement);
    dockWidgetInspector->show();
}

bool WebKitTabContent::canGoBack()
{
    webView->page()->history()->canGoBack();
}

bool WebKitTabContent::canGoForward()
{
    webView->page()->history()->canGoForward();
}
